import sqlalchemy as sa
from sqlalchemy import BIGINT, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import types, func
from src.libdata.settings import engine

Base = declarative_base()


def create_all():
    Base.metadata.create_all(bind=engine)


def drop_all():
    Base.metadata.drop_all()


def recreate_all():
    drop_all()
    create_all()


class BaseModel(Base):
    __abstract__ = True

    created_at = sa.Column(types.TIMESTAMP, server_default="NOW")
    updated_at = sa.Column(
        types.TIMESTAMP, server_default="NOW", onupdate=func.current_timestamp()
    )


class UserTable(BaseModel):
    __tablename__ = "users"

    id = sa.Column(BIGINT, primary_key=True, autoincrement=True)
    name = sa.Column(String, nullable=False)
    email = sa.Column(String, nullable=False)
    phone = sa.Column(String, nullable=True)



from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.engine import create_engine
from pydantic import BaseSettings, Field


class Settings(BaseSettings):
    database_url = Field(
        env="DATABASE_URL",
        default="postgresql://postgresUser:postgresPW@localhost:5432/postgresDB",
    )

    def get_engine(self):
        try:
            assert self.database_url
            return create_engine(self.database_url)
        except AssertionError as error:
            print(error)
        return None

    @staticmethod
    def get_session(db_engine):
        return scoped_session(
            sessionmaker(autocommit=True, autoflush=False, bind=db_engine)
        )


settings = Settings()
engine = settings.get_engine()
db_session = Settings.get_session(engine)


def get_session():
    db = db_session()
    try:
        yield db
    finally:
        db.close()
